import json

from django.dispatch import receiver
from django.conf import settings

from oscar.core.loading import import_module
product_viewed = import_module('catalogue.signals', ['product_viewed']).product_viewed

MAX_PRODUCTS = settings.OSCAR_RECENTLY_VIEWED_PRODUCTS

# Helpers

def get_recently_viewed_product_ids(request):
    u"""
    Returns the list of ids of the last products browsed by the user

    Limited to the max number defined in settings.py
    under OSCAR_RECENTLY_VIEWED_PRODUCTS.
    """
    try:
        product_ids = json.loads(request.COOKIES.get('oscar_recently_viewed_products'))
    except (ValueError, TypeError):
        return []
    if not isinstance(product_ids, list):
        return []
    return product_ids

def _update_recently_viewed_products(product, request, response):
    """
    Updates the cookies that store the recently viewed products
    removing possible duplicates.
    """
    product_ids = get_recently_viewed_product_ids(request)
    if product.id in product_ids:
        product_ids.remove(product.id)
    product_ids.append(product.id)
    if (len(product_ids) > MAX_PRODUCTS):
        product_ids = product_ids[len(product_ids)-MAX_PRODUCTS:]
    response.set_cookie('oscar_recently_viewed_products',
                        json.dumps(product_ids),
                        httponly=True)

# Receivers

@receiver(product_viewed)
def receive_product_view(sender, product, user, request, response, **kwargs):
    """
    Receiver to handle viewing single product pages

    Requires the request and response objects due to dependence on cookies
    """
    return _update_recently_viewed_products(product, request, response)



from decimal import Decimal as D
import httplib

from django.test import TestCase
from django.test.client import Client
from django.core.urlresolvers import reverse, clear_url_caches
from django.conf import settings

from oscar.test.helpers import create_product


class AnonCheckoutTests(TestCase):

    def setUp(self):
        clear_url_caches()
        self.client = Client()
        settings.OSCAR_ALLOW_ANON_CHECKOUT = True

    def tearDown(self):
        settings.OSCAR_ALLOW_ANON_CHECKOUT = False

    def _test_shipping_address_does_require_login(self):
        # Disabled until I can work out how to reload the URL config between tests
        url = reverse('checkout:shipping-address')
        response = self.client.get(url)
        self.assertEquals(httplib.OK, response.status_code)
    

class CheckoutViewsTest(TestCase):
    fixtures = ['example-shipping-charges.json']
    
    def setUp(self):
        clear_url_caches()
        self.client = Client()
        super(CheckoutViewsTest, self).setUp()
    
    def test_user_address_views_require_a_login(self):
        urls = [reverse('checkout:user-address-create'),
                reverse('checkout:user-address-update', kwargs={'pk': 1}),
                reverse('checkout:user-address-delete', kwargs={'pk': 1}),]
        for url in urls:
            response = self.client.get(url)
            self.assertEqual(httplib.FOUND, response.status_code)

    def test_anon_checkout_disabled_by_default(self):
        self.assertFalse(settings.OSCAR_ALLOW_ANON_CHECKOUT)

    def test_index_does_not_require_login(self):
        url = reverse('checkout:index')
        response = self.client.get(url)
        self.assertEquals(httplib.OK, response.status_code)

    def test_core_checkout_requires_login(self):
        urls = [reverse('checkout:shipping-address'),
                reverse('checkout:payment-method'),
                reverse('checkout:shipping-method'),
                reverse('checkout:payment-details')]
        for url in urls:
            response = self.client.get(url)
            self.assertEquals(httplib.FOUND, response.status_code)



from django import template
from django.db.models import get_model

from oscar.core.loading import import_module
forms = import_module('basket.forms', ['AddToBasketForm'])
Product = get_model('catalogue', 'product')

register = template.Library()


@register.tag(name="basket_form")
def do_basket_form(parse, token):
    """
    Template tag for adding the add-to-basket form to the
    template context so it can be rendered.
    """
    tokens = token.split_contents()
    if len(tokens) < 4 or tokens[3] != 'as':
        raise template.TemplateSyntaxError("%r tag uses the following syntax: {%% basket_form basket_var product_var as form_var %%}" % tokens[0])
    
    basket_var, product_var, form_var = tokens[1], tokens[2], tokens[4]
    return BasketFormNode(basket_var, product_var, form_var)


class BasketFormNode(template.Node):
    def __init__(self, basket_var, product_var, form_var):
        self.basket_var = template.Variable(basket_var)
        self.product_var = template.Variable(product_var)
        self.form_var = form_var
        
    def render(self, context):
        try:
            basket = self.basket_var.resolve(context)
            product = self.product_var.resolve(context)
        except template.VariableDoesNotExist:
            return ''
        
        if isinstance(product, Product):
            initial = {
                'product_id': product.id,
            }
            context[self.form_var] = forms.AddToBasketForm(basket, instance=product,initial=initial)
        return ''